
"""VirtualHostTool
"""

# requires the VirtualHostFolder product to be installed in the Zope root

from Products.CMFCore.utils import UniqueObject, getToolByName
from OFS.SimpleItem import SimpleItem
from Acquisition import aq_inner, aq_parent

from Globals import InitializeClass, DTMLFile, PersistentMapping
from AccessControl import ClassSecurityInfo, getSecurityManager
from Products.CMFCore import CMFCorePermissions
from utils import _dtmldir


# permissions are bad.  Fix later

class VirtualHostTool (UniqueObject, SimpleItem):
    id = 'portal_virtualhost'
    meta_type = 'CMF Virtual Host Tool'

    def __init__(self):
        self.vhfPath="/VHF" # holds path reference to virtual host folder (tool tries to find the VHF if that is not the id)

    security = ClassSecurityInfo()

    manage_options = ( { 'label' : 'Overview', 'action' : 'manage_overview' }
                     ,
                     ) + SimpleItem.manage_options

    high_security_names = ( 'CMF_manager_skins' , )

    medium_security_names = ( 'CMF_owner_skins' , 'CMF_default_skin' , 'Title' )

    secure_names = high_security_names + medium_security_names

    #
    #   ZMI methods
    #
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_overview' )
    manage_overview = DTMLFile( 'explainVirtualHostTool', _dtmldir )

    #
    #   'portal_virtualhost' interface methods
    #
    security.declarePublic( '__call__' )
    def __call__(self, request=None, *args, **kw):
        """
        Returns the title of the currently used virtual host, or the portal title .
        """
        return self.getTitle(request)

    def getTitle( self, request=None ):
        """ get the host or portal title """
        return self.getProperty( 'Title', request ) or getToolByName( self, 'portal_properties').title()

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getRootURL' )
    def getRootURL( self, request=None ):
        """
        Return the host's canonical root URL in regard to entire Zope
        """
        return self.getVirtualHostFolder().getCurrentVirtualHostRootURL( request )

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getActualRootURL' )
    def getActualRootURL( self, request=None ):
        """
        Return the host's actual current root URL in regard to entire Zope
        """
        return self.getVirtualHostFolder().getCurrentActualVirtualRootURL( request )

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getRootPath' )
    def getRootPath( self, request=None ):
        """
        Return the host's canonical root path in regard to entire Zope
        """
        return self.getRootURL( request ).split("/")[1:]

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getActualRootPath' )
    def getActualRootPath( self, request=None ):
        """
        Return the host's actual root path in regard to entire Zope
        """
        return self.getActualRootURL( request ).split("/")[1:]

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getRootObject' )
    def getRootObject( self, request = None ):
        """ Return the canonical root object """
        return self.getVirtualHostFolder().getCurrentVirtualHostRootObject(request)

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getActualRootObject' )
    def getActualRootObject( self, request = None ):
        """ Return the actual root object """
        return self.getVirtualHostFolder().getCurrentActualVirtualHostRootObject(request)

    security.declarePublic( 'getProperty' )
    def getProperty( self, attr, request=None ):
        """
        Returns the stored value of the given name, if any, for the current VH (or none).
        """
        vh= self.getVirtualHostFolder().getCurrentVirtualHostObject( request )
        return vh and vh.getProperty( attr ) or None


    security.declarePublic( 'getProperties' )
    def getProperties( self, request=None ):
        """
        Returns a list of property tuples (name, value) for the current VH.
        """
        vh = self.getVirtualHostFolder().getCurrentVirtualHostObject( request )
        return vh and [x for x in vh.propertyItems() if x[0] not in self.secure_names] or []

    security.declarePrivate( 'getVirtualHostFolder' )
    def getVirtualHostFolder( self ):
        result=None
        try:
            if not self.vhfPath: raise # so we can go to the except
            result=self.unrestrictedTraverse( self.vhfPath ) # will automatically raise an error and go to except if it fails
            if result.meta_type != "Virtual Host Folder": raise # so we can go to the except
        except:
            result=self.getPhysicalRoot().objectItems( "Virtual Host Folder" )
            if not result or not len(result):
                raise LookupError ("An object with 'Virtual Host Folder' meta_type is not found in Zope root.  The Zope instance must contain a VHF in the root to use the CMF Virtual Host Tool.")
            self.vhfPath="/"+result[0][0]
            result=result[0][1]
        return result

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getSkinManagerChoices' )
    def getSkinManagerChoices( self, request=None ):
        skinList= getToolByName( self, "portal_skins" ).getSkinSelections()
        result=[]
        managerList=self.getProperty( "CMF_manager_skins", request )
        if managerList:
            for skinName in managerList:
                if skinName in skinList: result.append(skinName)
        return result or skinList

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getSkinOwnerChoices' )
    def getSkinOwnerChoices( self, request=None ):
        skinList= self.getSkinManagerChoices()
        result=[]
        ownerList=self.getProperty( "CMF_owner_skins", request )
        if ownerList:
            for skinName in ownerList:
                if skinName in skinList: result.append(skinName)
        return result or skinList

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'getDefaultSkin' )
    def getDefaultSkinName( self, request=None ):
        result=self.getProperty( "CMF_default_skin", request )
        skinList=self.getSkinOwnerChoices( request )
        if result and result not in skinList: result = None
        if not result: result=getToolByName( self, "portal_skins" ).getDefaultSkin()
        if result not in skinList: result=skinList[0]
        return result

    # still lots of "set"ting to do too

    security.declarePrivate( 'validateHighSecurity' )
    def validateHighSecurity( self, request=None ):
        obj=self.getVirtualHostFolder().getCurrentVirtualHostRootObject( request )
        if not obj: return 0
        securityManager=getSecurityManager()
        return securityManager.checkPermission(
            CMFCorePermissions.ManagePortal, obj)

    security.declarePrivate( 'validateMediumSecurity' )
    def validateMediumSecurity( self, request=None ):
        obj=self.getVirtualHostFolder().getCurrentVirtualHostRootObject( request )
        if not obj: return 0
        securityManager=getSecurityManager()
        return securityManager.checkPermission(
            CMFCorePermissions.SetOwnProperties, obj)

    security.declarePrivate( 'validateBasicSecurity' )
    def validateBasicSecurity( self, request=None ):
        obj=self.getVirtualHostFolder().getCurrentVirtualHostRootObject( request )
        if not obj: return 0
        securityManager=getSecurityManager()
        return securityManager.checkPermission(
            CMFCorePermissions.ManageProperties, obj)

    security.declarePrivate( 'validate' )
    def validate (self, attr, request=None ):
        if attr in self.high_security_names:
            return self.validateHighSecurity( request )
        if attr in self.medium_security_names:
            return self.validateMediumSecurity( request )
        return self.validateBasicSecurity( request )

    security.declareProtected( CMFCorePermissions.AccessContentsInformation, 'setProperty' )
    def setProperty( self, attr, val, request=None ):
        """
        For host as indicated by hostkey, sets attr to val, unless it is one of the special skin properties.
        """
        if not self.validate( attr, request ):
            return 0
        self.getVirtualHostFolder().getCurrentHost( request ).manage_changeProperties({attr:val})
        return 1


    security.declarePublic( 'getBreadcrumbs' )
    def getBreadcrumbs(self, request=None, rootName="home"):
        """ list of name, link lists """
        return self.getVirtualHostFolder().getCleanOriginalRequestList(request, rootName)

    security.declarePublic( 'getOriginalRequestURL' )
    def getOriginalRequestURL(self, request=None):
        """ the path part of the URL as originally entered """
        return self.getVirtualHostFolder().getOriginalRequestURL(request)

    security.declarePublic( 'getPathToContent' )
    def getPathToContent(self, obj, request=None):
        """ get the path to an object, taking into account aliases and the
        like """
        return self.getVirtualHostFolder().getVirtualPathToObject( obj, request)

    security.declarePublic( 'getURLToContent' )
    def getURLToContent(self, obj, request=None):
        """ get the URL to an object, taking into account aliases and the
        like """
        return self.getVirtualHostFolder().getVirtualURLToObject( obj, request)

    security.declarePublic( 'getStatus' )
    def getStatus(self, request=None):
        """ get 1 (virtual host in use in current request) or 0 (nope) """
        return self.getVirtualHostFolder().getVirtualHostStatus(request)

InitializeClass(VirtualHostTool)
